package matt.implementation;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeSet;

import matt.adapters.MessageSource;
import matt.protocol.Runtime;

public class DirectorySource
  implements
    MessageSource {

  private final File directory;
  private final FilenameFilter filter;
  private List<String> messages = new LinkedList<String>();

  private class FileSorter
    implements
      Comparator<File> {

    public int compare(
      File lhs,
      File rhs) {
      if (lhs.lastModified() < rhs.lastModified()) {
        return -1;
      }
      if (lhs.lastModified() > rhs.lastModified()) {
        return 1;
      }
      return 0;
    }
  }

  class NewFileFilter
    implements
      FilenameFilter {

    private final String extension;
    private long previousTimestamp = 0;

    public NewFileFilter(
      final String extension) {
      this.extension = extension;
    }

    private boolean accept(
      File file) {

      if (file.lastModified() >= previousTimestamp) {
        if (file.getName().endsWith(extension)) {
          return true;
        }
      }
      return false;
    }

    public boolean accept(
      File dir,
      String name) {
      return accept(new File(dir.getPath(), name));
    }

  }

  public DirectorySource(
    final Runtime runtime) {
    final String path = runtime.getProperty(
      "Matt.DirectorySource.Path",
      "messages");
    final String fileExt = "."
        + runtime.getProperty("Matt.DirectorySource.MessageExtension", "msg");
    filter = new NewFileFilter(fileExt);
    directory = new File(path);
    if (directory.exists()) {
      if (!directory.isDirectory()) {
        throw new RuntimeException(path
            + " already exists and is not a directory");
      }
    }
    else {
      directory.mkdirs();
    }
  }

  public String getMessage() {
    synchronized (messages) {
      return messages.remove(0);
    }
  }

  public boolean hasMessage() {
    synchronized (messages) {
      if (messages.size() == 0) {
        TreeSet<File> files = new TreeSet<File>(new FileSorter());
        files.addAll(Arrays.asList(directory.listFiles(filter)));
        if (files.size() > 0) {
          try {
            for (final File file : files) {
              final InputStream stream = new FileInputStream(file);
              final InputStreamReader reader = new InputStreamReader(stream);
              final char[] bytes = new char[(int) file.length()];
              reader.read(bytes);
              reader.close();
              messages.add(new String(bytes));
              if (!file.delete()) {
                throw new RuntimeException("Could not delete file "
                    + file.getAbsolutePath());
              }
            }
          }
          catch (IOException e) {
            // TODO
            e.printStackTrace();
          }
        }
      }
      return messages.size() != 0;
    }
  }

  public void bubleException(
    Exception e) {
    // TODO Auto-generated method stub
  }

}
